/*
 * Semaphore utilities.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/sys/err.h>
#include <l4/sys/semaphore.h>

#include "semaphore.h"



/* Obtain control of a semaphore. */

long ipc_semaphore_down(l4_cap_idx_t semaphore)
{
  long err;

  if (l4_is_valid_cap(semaphore))
  {
    do
    {
      /* Since l4_irq_trigger is a "send-only" operation, l4_error is not
         appropriate. */

      err = l4_ipc_error(l4_semaphore_down(semaphore, L4_IPC_NEVER), l4_utcb());

      /* Handle EAGAIN ("Resource temporarily unavailable") by just retrying. */

      if (err)
      {
        if (err == -L4_EAGAIN)
          continue;
        else
          return err;
      }
    }
    while (0);
  }

  return L4_EOK;
}

/* Relinquish control of a semaphore. */

long ipc_semaphore_up(l4_cap_idx_t semaphore)
{
  long err;

  if (l4_is_valid_cap(semaphore))
  {
    do
    {
      /* Since l4_irq_trigger is a "send-only" operation, l4_error is not
         appropriate. */

      err = l4_ipc_error(l4_semaphore_up(semaphore), l4_utcb());

      /* Handle EAGAIN ("Resource temporarily unavailable") by just retrying. */

      if (err)
      {
        if (err == -L4_EAGAIN)
          continue;
        else
          return err;
      }
    }
    while (0);
  }

  return L4_EOK;
}

// vim: tabstop=2 expandtab shiftwidth=2
