/*
 * Memory region abstractions.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <systypes/base.h>

#include "types.h"



/* Region-related state information. */

class RegionState
{
public:
    fileid_t fileid;
    offset_t filepos;

    explicit RegionState(fileid_t fileid=0, offset_t filepos=0);

    void fill(fileid_t fileid, offset_t filepos);

    void flush();
};



/* A memory region abstraction. */

class Region
{
public:
    offset_t start, end;

    /* Debugging information. */

    RegionState state;

    /* Methods. */

    explicit Region(offset_t start, offset_t end);

    virtual ~Region();

    offset_t size();

    int compare(Region *other);

    void fill(fileid_t fileid, offset_t filepos);

    void flush();

    /* Simulation methods. */

    char *read(offset_t offset=0);

    void write(const char *data, offset_t offset=0);
};

// vim: tabstop=4 expandtab shiftwidth=4
